@@ -3,7 +3,7 @@
 # Distributed under the terms of the Modified BSD License.
 from tornado import web
 
-from jupyter_server.auth import authorized
+from jupyter_server.auth.decorator import authorized
 
 from ...base.handlers import APIHandler
 from . import csp_report_uri
