@@ -1,90 +1,34 @@
 """Tests for authorization"""
 import json
+import os
 
 import pytest
 from jupyter_client.kernelspec import NATIVE_KERNEL_NAME
 from nbformat import writes
 from nbformat.v4 import new_notebook
-from tornado.httpclient import HTTPClientError
-from tornado.websocket import WebSocketHandler
 
-from jupyter_server.auth.authorizer import Authorizer
-from jupyter_server.auth.utils import HTTP_METHOD_TO_AUTH_ACTION, match_url_to_resource
 from jupyter_server.services.security import csp_report_uri
 
 
-class AuthorizerforTesting(Authorizer):
-
-    # Set these class attributes from within a test
-    # to verify that they match the arguments passed
-    # by the REST API.
-    permissions = {}
-
-    def normalize_url(self, path):
-        """Drop the base URL and make sure path leads with a /"""
-        base_url = self.parent.base_url
-        # Remove base_url
-        if path.startswith(base_url):
-            path = path[len(base_url) :]
-        # Make sure path starts with /
-        if not path.startswith("/"):
-            path = "/" + path
-        return path
-
-    def is_authorized(self, handler, user, action, resource):
-        # Parse Request
-        if isinstance(handler, WebSocketHandler):
-            method = "WEBSOCKET"
-        else:
-            method = handler.request.method
-        url = self.normalize_url(handler.request.path)
-
-        # Map request parts to expected action and resource.
-        expected_action = HTTP_METHOD_TO_AUTH_ACTION[method]
-        expected_resource = match_url_to_resource(url)
-
-        # Assert that authorization layer returns the
-        # correct action + resource.
-        assert action == expected_action
-        assert resource == expected_resource
-
-        # Now, actually apply the authorization layer.
-        return all(
-            [
-                action in self.permissions.get("actions", []),
-                resource in self.permissions.get("resources", []),
-            ]
-        )
+@pytest.fixture()
+def jp_server_config(jp_server_authorizer):
+    return {
+        "ServerApp": {"authorizer_class": jp_server_authorizer},
+        "jpserver_extensions": {"jupyter_server_terminals": True},
+    }
 
 
-@pytest.fixture
-def jp_server_config():
-    return {"ServerApp": {"authorizer_class": AuthorizerforTesting}}
-
-
-@pytest.fixture
-def send_request(jp_fetch, jp_ws_fetch):
-    """Send to Jupyter Server and return response code."""
-
-    async def _(url, **fetch_kwargs):
-        if url.endswith("channels") or "/websocket/" in url:
-            fetch = jp_ws_fetch
-        else:
-            fetch = jp_fetch
-
-        try:
-            r = await fetch(url, **fetch_kwargs, allow_nonstandard_methods=True)
-            code = r.code
-        except HTTPClientError as err:
-            code = err.code
-        else:
-            if fetch is jp_ws_fetch:
-                r.close()
-
-        print(code, url, fetch_kwargs)
-        return code
-
-    return _
+@pytest.fixture()
+def jp_server_auth_resources(jp_server_auth_core_resources):
+    # terminal plugin doesn't have importable url patterns
+    # get these from terminal/__init__.py
+    for url_regex in [
+        r"/terminals/websocket/(\w+)",
+        "/api/terminals",
+        r"/api/terminals/(\w+)",
+    ]:
+        jp_server_auth_core_resources[url_regex] = "terminals"
+    return jp_server_auth_core_resources
 
 
 HTTP_REQUESTS = [
@@ -205,7 +149,6 @@ async def test_authorized_requests(
     send_request,
     tmp_path,
     jp_serverapp,
-    jp_cleanup_subprocesses,
     method,
     url,
     body,
@@ -228,12 +171,12 @@ async def test_authorized_requests(
     km = jp_serverapp.kernel_manager
 
     if "session" in url:
-        request.addfinalizer(lambda: io_loop.run_sync(km.shutdown_all))
         session_model = await jp_serverapp.session_manager.create_session(path="foo")
         session_id = session_model["id"]
 
     if "kernel" in url:
-        request.addfinalizer(lambda: io_loop.run_sync(km.shutdown_all))
+        if os.name == "nt":
+            pytest.skip("Test hangs on Windows")
         kernel_id = await km.start_kernel()
         kernel = km.get_kernel(kernel_id)
         # kernels take a moment to be ready
@@ -245,7 +188,6 @@ async def test_authorized_requests(
 
     if "terminal" in url:
         term_manager = jp_serverapp.web_app.settings["terminal_manager"]
-        request.addfinalizer(lambda: io_loop.run_sync(term_manager.terminate_all))
         term_model = term_manager.create()
         term_name = term_model["name"]
 
@@ -275,5 +217,3 @@ async def test_authorized_requests(
 
     code = await send_request(url, body=body, method=method)
     assert code in expected_codes
-
-    await jp_cleanup_subprocesses()
