@@ -82,7 +82,7 @@ async def test_list_nonexistant_dir(jp_fetch, contents):
         await jp_fetch(
             "api",
             "contents",
-            "nonexistant",
+            "nonexistent",
             method="GET",
         )
 
@@ -102,6 +102,20 @@ async def test_get_nb_contents(jp_fetch, contents, path, name):
     assert isinstance(model["content"]["metadata"], dict)
 
 
+@pytest.mark.parametrize("path,name", dirs)
+async def test_get_nb_md5(jp_fetch, contents, path, name):
+    nbname = name + ".ipynb"
+    nbpath = (path + "/" + nbname).lstrip("/")
+    r = await jp_fetch("api", "contents", nbpath, method="GET", params=dict(md5="1"))
+    model = json.loads(r.body.decode())
+    assert model["name"] == nbname
+    assert model["path"] == nbpath
+    assert model["type"] == "notebook"
+    assert "md5" in model
+    assert "metadata" in model["content"]
+    assert isinstance(model["content"]["metadata"], dict)
+
+
 @pytest.mark.parametrize("path,name", dirs)
 async def test_get_nb_no_contents(jp_fetch, contents, path, name):
     nbname = name + ".ipynb"
@@ -186,6 +200,19 @@ async def test_get_text_file_contents(jp_fetch, contents, path, name):
     assert expected_http_error(e, 400)
 
 
+@pytest.mark.parametrize("path,name", dirs)
+async def test_get_text_file_md5(jp_fetch, contents, path, name):
+    txtname = name + ".txt"
+    txtpath = (path + "/" + txtname).lstrip("/")
+    r = await jp_fetch("api", "contents", txtpath, method="GET", params=dict(md5="1"))
+    model = json.loads(r.body.decode())
+    assert model["name"] == txtname
+    assert model["path"] == txtpath
+    assert "md5" in model
+    assert model["format"] == "text"
+    assert model["type"] == "file"
+
+
 async def test_get_404_hidden(jp_fetch, contents, contents_dir):
     # Create text files
     hidden_dir = contents_dir / ".hidden"
@@ -268,7 +295,7 @@ async def test_get_bad_type(jp_fetch, contents):
     assert expected_http_error(e, 400, "%s is not a directory" % path)
 
 
-@pytest.fixture
+@pytest.fixture()
 def _check_created(jp_base_url):
     def _inner(r, contents_dir, path, name, type="notebook"):
         fpath = path + "/" + name
