@@ -67,9 +67,9 @@ def test_atomic_writing(tmp_path):
             assert f.read() == "written from symlink"
 
 
-@pytest.fixture
+@pytest.fixture()
 def handle_umask():
-    global umask  # noqa
+    global umask
     umask = os.umask(0)
     os.umask(umask)
     yield
@@ -139,9 +139,11 @@ def test_path_to_invalid(tmpdir):
 @pytest.mark.skipif(os.name == "nt", reason="test fails on Windows")
 def test_file_manager_mixin(tmpdir):
     mixin = FileManagerMixin()
-    mixin.log = logging.getLogger()  # type:ignore
+    mixin.log = logging.getLogger()
     bad_content = tmpdir / "bad_content.ipynb"
     bad_content.write_text("{}", "utf8")
+    # Same as `echo -n {} | md5sum`
+    assert mixin._get_md5(bad_content) == "99914b932bd37a50b983c5e7c90ae93b"
     with pytest.raises(HTTPError):
         mixin._read_notebook(bad_content)
     other = path_to_intermediate(bad_content)
@@ -161,9 +163,11 @@ def test_file_manager_mixin(tmpdir):
 @pytest.mark.skipif(os.name == "nt", reason="test fails on Windows")
 async def test_async_file_manager_mixin(tmpdir):
     mixin = AsyncFileManagerMixin()
-    mixin.log = logging.getLogger()  # type:ignore
+    mixin.log = logging.getLogger()
     bad_content = tmpdir / "bad_content.ipynb"
     bad_content.write_text("{}", "utf8")
+    # Same as `echo -n {} | md5sum`
+    assert await mixin._get_md5(bad_content) == "99914b932bd37a50b983c5e7c90ae93b"
     with pytest.raises(HTTPError):
         await mixin._read_notebook(bad_content)
     other = path_to_intermediate(bad_content)
