@@ -296,7 +296,7 @@ async def test_403(jp_file_contents_manager_class, tmp_path):
         assert e.status_code == 403
 
 
-async def test_400(jp_file_contents_manager_class, tmp_path):  # noqa
+async def test_400(jp_file_contents_manager_class, tmp_path):
     # Test Delete behavior
     # Test delete of file in hidden directory
     td = str(tmp_path)
@@ -406,35 +406,44 @@ async def test_400(jp_file_contents_manager_class, tmp_path):  # noqa
 
 
 async def test_404(jp_file_contents_manager_class, tmp_path):
-    # Test visible file in hidden folder
-    with pytest.raises(HTTPError) as excinfo:
-        td = str(tmp_path)
-        cm = jp_file_contents_manager_class(root_dir=td)
-        hidden_dir = ".hidden"
-        file_in_hidden_path = os.path.join(hidden_dir, "visible.txt")
-        _make_dir(cm, hidden_dir)
-        model = await ensure_async(cm.new(path=file_in_hidden_path))
-        os_path = cm._get_os_path(model["path"])
+    # setup
+    td = str(tmp_path)
+    cm = jp_file_contents_manager_class(root_dir=td)
 
-        try:
-            result = await ensure_async(cm.get(os_path, "w"))
-        except HTTPError as e:
-            assert e.status_code == 404
+    # Test visible file in hidden folder
+    cm.allow_hidden = True
+    hidden_dir = ".hidden"
+    file_in_hidden_path = os.path.join(hidden_dir, "visible.txt")
+    _make_dir(cm, hidden_dir)
+    model = await ensure_async(cm.new(path=file_in_hidden_path))
+    os_path = cm._get_os_path(model["path"])
+    cm.allow_hidden = False
+
+    with pytest.raises(HTTPError) as excinfo:
+        await ensure_async(cm.get(os_path))
+        assert excinfo.value.status_code == 404
 
     # Test hidden file in visible folder
-    with pytest.raises(HTTPError) as excinfo:
-        td = str(tmp_path)
-        cm = jp_file_contents_manager_class(root_dir=td)
-        hidden_dir = "visible"
-        file_in_hidden_path = os.path.join(hidden_dir, ".hidden.txt")
-        _make_dir(cm, hidden_dir)
-        model = await ensure_async(cm.new(path=file_in_hidden_path))
-        os_path = cm._get_os_path(model["path"])
+    cm.allow_hidden = True
+    hidden_dir = "visible"
+    file_in_hidden_path = os.path.join(hidden_dir, ".hidden.txt")
+    _make_dir(cm, hidden_dir)
+    model = await ensure_async(cm.new(path=file_in_hidden_path))
+    os_path = cm._get_os_path(model["path"])
+    cm.allow_hidden = False
 
-        try:
-            result = await ensure_async(cm.get(os_path, "w"))
-        except HTTPError as e:
-            assert e.status_code == 404
+    with pytest.raises(HTTPError) as excinfo:
+        await ensure_async(cm.get(os_path))
+        assert excinfo.value.status_code == 404
+
+    # Test file not found
+    td = str(tmp_path)
+    cm = jp_file_contents_manager_class(root_dir=td)
+    not_a_file = "foo.bar"
+
+    with pytest.raises(HTTPError) as excinfo:
+        await ensure_async(cm.get(not_a_file))
+        assert excinfo.value.status_code == 404
 
 
 async def test_escape_root(jp_file_contents_manager_class, tmp_path):
@@ -448,19 +457,19 @@ async def test_escape_root(jp_file_contents_manager_class, tmp_path):
 
     with pytest.raises(HTTPError) as e:
         await ensure_async(cm.get(".."))
-    expected_http_error(e, 404)
+    assert expected_http_error(e, 404)
 
     with pytest.raises(HTTPError) as e:
         await ensure_async(cm.get("foo/../../../bar"))
-    expected_http_error(e, 404)
+    assert expected_http_error(e, 404)
 
     with pytest.raises(HTTPError) as e:
         await ensure_async(cm.delete("../foo"))
-    expected_http_error(e, 404)
+    assert expected_http_error(e, 404)
 
     with pytest.raises(HTTPError) as e:
         await ensure_async(cm.rename("../foo", "../bar"))
-    expected_http_error(e, 404)
+    assert expected_http_error(e, 404)
 
     with pytest.raises(HTTPError) as e:
         await ensure_async(
@@ -473,7 +482,7 @@ async def test_escape_root(jp_file_contents_manager_class, tmp_path):
                 path="../foo",
             )
         )
-    expected_http_error(e, 404)
+    assert expected_http_error(e, 404)
 
 
 async def test_new_untitled(jp_contents_manager):
@@ -538,7 +547,7 @@ async def test_modified_date(jp_contents_manager):
     assert renamed["last_modified"] >= saved["last_modified"]
 
 
-async def test_get(jp_contents_manager):  # noqa
+async def test_get(jp_contents_manager):
     cm = jp_contents_manager
     # Create a notebook
     model = await ensure_async(cm.new_untitled(type="notebook"))
@@ -562,6 +571,9 @@ async def test_get(jp_contents_manager):  # noqa
     nb_as_bin_file = await ensure_async(cm.get(path, content=True, type="file", format="base64"))
     assert nb_as_bin_file["format"] == "base64"
 
+    nb_with_md5 = await ensure_async(cm.get(path, md5=True))
+    assert nb_with_md5["md5"]
+
     # Test in sub-directory
     sub_dir = "/foo/"
     _make_dir(cm, "foo")
@@ -576,7 +588,7 @@ async def test_get(jp_contents_manager):  # noqa
 
     # Test with a regular file.
     file_model_path = (await ensure_async(cm.new_untitled(path=sub_dir, ext=".txt")))["path"]
-    file_model = await ensure_async(cm.get(file_model_path))
+    file_model = await ensure_async(cm.get(file_model_path, md5=True))
     expected_model = {
         "content": "",
         "format": "text",
@@ -591,6 +603,7 @@ async def test_get(jp_contents_manager):  # noqa
         assert file_model[key] == value
     assert "created" in file_model
     assert "last_modified" in file_model
+    assert "md5" in file_model
 
     # Create a sub-sub directory to test getting directory contents with a
     # subdir.
