@@ -9,22 +9,29 @@ from email.utils import format_datetime
 from http.cookies import SimpleCookie
 from io import BytesIO
 from queue import Empty
-from typing import Any, Union
+from typing import Any, Dict, Union
 from unittest.mock import MagicMock, patch
 
 import pytest
 import tornado
 from jupyter_core.utils import ensure_async
+from tornado.concurrent import Future
 from tornado.httpclient import HTTPRequest, HTTPResponse
+from tornado.httputil import HTTPServerRequest
+from tornado.queues import Queue
 from tornado.web import HTTPError
 from traitlets import Int, Unicode
 from traitlets.config import Config
 
+from jupyter_server.gateway.connections import GatewayWebSocketConnection
 from jupyter_server.gateway.gateway_client import GatewayTokenRenewerBase, NoOpTokenRenewer
 from jupyter_server.gateway.managers import ChannelQueue, GatewayClient, GatewayKernelManager
+from jupyter_server.services.kernels.websocket import KernelWebsocketHandler
 
 from .utils import expected_http_error
 
+pytest_plugins = ["jupyter_events.pytest_plugin"]
+
 
 def generate_kernelspec(name):
     argv_stanza = ["python", "-m", "ipykernel_launcher", "-f", "{connection_file}"]
@@ -62,10 +69,16 @@ kernelspecs: dict = {
 # maintain a dictionary of expected running kernels.  Key = kernel_id, Value = model.
 running_kernels = {}
 
+# Dictionary of kernels to transiently omit from list results.
+#
+# This is used to simulate inconsistency in list results from the Gateway server
+# due to issues like race conditions, bugs, etc.
+omitted_kernels: Dict[str, bool] = {}
+
 
 def generate_model(name):
     """Generate a mocked kernel model.  Caller is responsible for adding model to running_kernels dictionary."""
-    dt = datetime.utcnow().isoformat() + "Z"  # noqa
+    dt = datetime.now(timezone.utc).isoformat().replace("+00:00", "Z")
     kernel_id = str(uuid.uuid4())
     model = {
         "id": kernel_id,
@@ -77,7 +90,7 @@ def generate_model(name):
     return model
 
 
-async def mock_gateway_request(url, **kwargs):  # noqa
+async def mock_gateway_request(url, **kwargs):
     method = "GET"
     if kwargs["method"]:
         method = kwargs["method"]
@@ -131,8 +144,11 @@ async def mock_gateway_request(url, **kwargs):  # noqa
     if endpoint.endswith("/api/kernels") and method == "GET":
         kernels = []
         for kernel_id in running_kernels:
-            model = running_kernels.get(kernel_id)
-            kernels.append(model)
+            if kernel_id in omitted_kernels:
+                omitted_kernels.pop(kernel_id)
+            else:
+                model = running_kernels.get(kernel_id)
+                kernels.append(model)
         response_buf = BytesIO(json.dumps(kernels).encode("utf-8"))
         response = await ensure_async(HTTPResponse(request, 200, buffer=response_buf))
         return response
@@ -201,9 +217,9 @@ class CustomTestTokenRenewer(GatewayTokenRenewerBase):  # type:ignore[misc]
 
     # The following are configured by the config test to ensure they flow
     # configured to: 42
-    config_var_1: int = Int(config=True)  # type:ignore
+    config_var_1: int = Int(config=True)  # type:ignore[assignment]
     # configured to: "Use this token value: "
-    config_var_2: str = Unicode(config=True)  # type:ignore
+    config_var_2: str = Unicode(config=True)  # type:ignore[assignment]
 
     def get_token(
         self, auth_header_key: str, auth_scheme: Union[str, None], auth_token: str, **kwargs: Any
@@ -218,7 +234,7 @@ def jp_server_config():
     )
 
 
-@pytest.fixture
+@pytest.fixture()
 def init_gateway(monkeypatch):
     """Initializes the server for use as a gateway client."""
     # Clear the singleton first since previous tests may not have used a gateway.
@@ -287,13 +303,19 @@ def test_gateway_cli_options(jp_configurable_serverapp, capsys):
     GatewayClient.clear_instance()
 
 
-@pytest.mark.parametrize("renewer_type", ["default", "custom"])
-def test_token_renewer_config(jp_server_config, jp_configurable_serverapp, renewer_type):
+@pytest.mark.parametrize(
+    "renewer_type,initial_auth_token", [("default", ""), ("custom", None), ("custom", "")]
+)
+def test_token_renewer_config(
+    jp_server_config, jp_configurable_serverapp, renewer_type, initial_auth_token
+):
     argv = ["--gateway-url=" + mock_gateway_url]
     if renewer_type == "custom":
         argv.append(
             "--GatewayClient.gateway_token_renewer_class=tests.test_gateway.CustomTestTokenRenewer"
         )
+    if initial_auth_token is None:
+        argv.append("--GatewayClient.auth_token=None")
 
     GatewayClient.clear_instance()
     app = jp_configurable_serverapp(argv=argv)
@@ -306,15 +328,20 @@ def test_token_renewer_config(jp_server_config, jp_configurable_serverapp, renew
     if renewer_type == "default":
         assert isinstance(gw_client.gateway_token_renewer, NoOpTokenRenewer)
         token = gw_client.gateway_token_renewer.get_token(
-            gw_client.auth_header_key, gw_client.auth_scheme, gw_client.auth_token
+            gw_client.auth_header_key, gw_client.auth_scheme, gw_client.auth_token or ""
         )
         assert token == gw_client.auth_token
     else:
         assert isinstance(gw_client.gateway_token_renewer, CustomTestTokenRenewer)
         token = gw_client.gateway_token_renewer.get_token(
-            gw_client.auth_header_key, gw_client.auth_scheme, gw_client.auth_token
+            gw_client.auth_header_key, gw_client.auth_scheme, gw_client.auth_token or ""
         )
         assert token == CustomTestTokenRenewer.TEST_EXPECTED_TOKEN_VALUE
+    gw_client.load_connection_args()
+    if renewer_type == "default" or initial_auth_token is None:
+        assert gw_client.auth_token == initial_auth_token
+    else:
+        assert gw_client.auth_token == CustomTestTokenRenewer.TEST_EXPECTED_TOKEN_VALUE
 
 
 @pytest.mark.parametrize(
@@ -453,6 +480,7 @@ async def test_gateway_session_lifecycle(init_gateway, jp_root_dir, jp_fetch, cu
 
     assert await is_session_active(jp_fetch, session_id) is True
 
+    omitted_kernels[kernel_id] = True
     if cull_kernel:
         running_kernels.pop(kernel_id)
 
@@ -473,13 +501,28 @@ async def test_gateway_session_lifecycle(init_gateway, jp_root_dir, jp_fetch, cu
 
 @pytest.mark.parametrize("cull_kernel", [False, True])
 async def test_gateway_kernel_lifecycle(
-    init_gateway, jp_serverapp, jp_ws_fetch, jp_fetch, cull_kernel
+    init_gateway,
+    jp_configurable_serverapp,
+    jp_read_emitted_events,
+    jp_event_handler,
+    jp_ws_fetch,
+    jp_fetch,
+    cull_kernel,
 ):
     # Validate kernel lifecycle functions; create, interrupt, restart and delete.
 
+    app = jp_configurable_serverapp()
+    app.event_logger.register_handler(jp_event_handler)
+
     # create
     kernel_id = await create_kernel(jp_fetch, "kspec_bar")
 
+    output = jp_read_emitted_events()[0]
+    assert "action" in output and output["action"] == "start"
+    assert "msg" in output
+    assert "kernel_id" in output and kernel_id == output["kernel_id"]
+    assert "status" in output and output["status"] == "success"
+
     # ensure kernel still considered running
     assert await is_kernel_running(jp_fetch, kernel_id) is True
 
@@ -492,15 +535,28 @@ async def test_gateway_kernel_lifecycle(
     # interrupt
     await interrupt_kernel(jp_fetch, kernel_id)
 
+    output = jp_read_emitted_events()[0]
+    assert "action" in output and output["action"] == "interrupt"
+    assert "msg" in output
+    assert "kernel_id" in output and kernel_id == output["kernel_id"]
+    assert "status" in output and output["status"] == "success"
+
     # ensure kernel still considered running
     assert await is_kernel_running(jp_fetch, kernel_id) is True
 
     # restart
     await restart_kernel(jp_fetch, kernel_id)
 
+    output = jp_read_emitted_events()[0]
+    assert "action" in output and output["action"] == "restart"
+    assert "msg" in output
+    assert "kernel_id" in output and kernel_id == output["kernel_id"]
+    assert "status" in output and output["status"] == "success"
+
     # ensure kernel still considered running
     assert await is_kernel_running(jp_fetch, kernel_id) is True
 
+    omitted_kernels[kernel_id] = True
     if cull_kernel:
         running_kernels.pop(kernel_id)
 
@@ -515,6 +571,12 @@ async def test_gateway_kernel_lifecycle(
     else:
         await delete_kernel(jp_fetch, kernel_id)
 
+        output = jp_read_emitted_events()[0]
+        assert "action" in output and output["action"] == "shutdown"
+        assert "msg" in output
+        assert "kernel_id" in output and kernel_id == output["kernel_id"]
+        assert "status" in output and output["status"] == "success"
+
     assert await is_kernel_running(jp_fetch, kernel_id) is False
 
 
@@ -566,7 +628,7 @@ async def test_kernel_client_response_router_notifies_channel_queue_when_finishe
         kc.hb_channel,
         kc.control_channel,
     ]
-    assert all(channel.response_router_finished if True else False for channel in all_channels)
+    assert all(channel.response_router_finished for channel in all_channels)
 
     await ensure_async(kc.stop_channels())
 
@@ -613,6 +675,61 @@ async def test_channel_queue_get_msg_when_response_router_had_finished():
         await queue.get_msg()
 
 
+class MockWebSocketClientConnection(tornado.websocket.WebSocketClientConnection):
+    def __init__(self, *args, **kwargs):
+        self._msgs: Queue = Queue(2)
+        self._msgs.put_nowait('{"msg_type": "status", "content": {"execution_state": "starting"}}')
+
+    def write_message(self, message, *args, **kwargs):
+        return self._msgs.put(message)
+
+    def read_message(self, *args, **kwargs):
+        return self._msgs.get()
+
+
+def mock_websocket_connect():
+    def helper(request):
+        fut: Future = Future()
+        mock_client = MockWebSocketClientConnection()
+        fut.set_result(mock_client)
+        return fut
+
+    return helper
+
+
+@patch("tornado.websocket.websocket_connect", mock_websocket_connect())
+async def test_websocket_connection_closed(init_gateway, jp_serverapp, jp_fetch, caplog):
+    # Create the kernel and get the kernel manager...
+    kernel_id = await create_kernel(jp_fetch, "kspec_foo")
+    km: GatewayKernelManager = jp_serverapp.kernel_manager.get_kernel(kernel_id)
+
+    # Create the KernelWebsocketHandler...
+    request = HTTPServerRequest("foo", "GET")
+    request.connection = MagicMock()
+    handler = KernelWebsocketHandler(jp_serverapp.web_app, request)
+
+    # Force the websocket handler to raise a closed error if we try to write a message
+    # to the client.
+    handler.ws_connection = MagicMock()
+    handler.ws_connection.is_closing = lambda: True
+
+    # Create the GatewayWebSocketConnection and attach it to the handler...
+    conn = GatewayWebSocketConnection(parent=km, websocket_handler=handler)
+    handler.connection = conn
+    await conn.connect()
+
+    # Processing websocket messages happens in separate coroutines and any
+    # errors in that process will show up in logs, but not bubble up to the
+    # caller.
+    #
+    # To check for these, we wait for the server to stop and then check the
+    # logs for errors.
+    await jp_serverapp._cleanup()
+    for _, level, message in caplog.record_tuples:
+        if level >= logging.ERROR:
+            pytest.fail(f"Logs contain an error: {message}")
+
+
 #
 # Test methods below...
 #
@@ -681,7 +798,7 @@ async def is_kernel_running(jp_fetch, kernel_id):
 
 
 async def create_kernel(jp_fetch, kernel_name):
-    """Issues request to retart the given kernel"""
+    """Issues request to restart the given kernel"""
     with mocked_gateway:
         body = json.dumps({"name": kernel_name})
 
@@ -719,7 +836,7 @@ async def interrupt_kernel(jp_fetch, kernel_id):
 
 
 async def restart_kernel(jp_fetch, kernel_id):
-    """Issues request to retart the given kernel"""
+    """Issues request to restart the given kernel"""
     with mocked_gateway:
         r = await jp_fetch(
             "api",
